@@ -20,7 +20,7 @@
  *
  */
 
-/* $Id: main.c,v 1.78.2.2 2005/11/14 12:48:50 nyaochi Exp $ */
+/* $Id: main.c,v 1.90 2006/05/02 04:56:12 nyaochi Exp $ */
 
 #ifdef	HAVE_CONFIG_H
 #include <config.h>
@@ -47,12 +47,13 @@
 #include "getopt.h"
 #endif
 
-#define	APPLICATION_S	"EasyH10 [CUI]"
-#define	VERSION_S		"1.2.1"
-#define	COPYRIGHT_S		"Copyright (c) 2005 by Nyaochi"
+#define	APPLICATION_S		"EasyH10 [CUI]"
+#define	VERSION_S			"1.4"
+#define	COPYRIGHT_S			"Copyright (c) 2005-2006 by Nyaochi"
 
 #ifndef		_WIN32
-#define	MBS_CHARSET	"UTF-8"
+#define	SYSTEM_ENCODING		"UTF-8"
+#define	MUSICTAG_ENCODING	"ISO-8859-1"
 #endif
 
 static int verbose = 0;
@@ -66,6 +67,8 @@ enum {
 	MODE_DUMP =			0x00000010,
 	MODE_HELP =			0x00000020,
 	MODE_VERSION =		0x00000040,
+	MODE_IMPORT =		0x00000100,
+	MODE_EXPORT =		0x00000200,
 };
 
 void usage(FILE *fp, const char *argv0)
@@ -106,6 +109,13 @@ void usage(FILE *fp, const char *argv0)
 	fprintf(fp, "  -g, --guess-missing   Guess pathnames of missing music files in a playlist\n");
 	fprintf(fp, "  -f, --find-missing    Find pathnames of missing music files in a playlist\n");
 	fprintf(fp, "  -k, --skip-missing    Skip missing music files in a playlist and continue\n");
+	fprintf(fp, "  -x, --regex-playlist  Enable Regular Expression Playlist (REP)\n");
+	fprintf(fp, "  -l, --auto-shuffle    Shuffle music files in [(name).sfl*.(ext)] playlists\n");
+	fprintf(fp, "\n");
+	fprintf(fp, "Tuner preset options:\n");
+	fprintf(fp, "  -I, --import-tuner=FN Import a tuner preset from a file\n");
+	fprintf(fp, "  -E, --export-tuner=FN Export the current tuner preset to a file\n");
+	fprintf(fp, "  -a, --merge-tuner     Merge the current tuner preset with a file\n");
 	fprintf(fp, "\n");
 	fprintf(fp, "H10 player model options:\n");
 	fprintf(fp, "  -t, --model=MODEL     Apply a model template for specifying an H10 model\n");
@@ -119,14 +129,12 @@ void usage(FILE *fp, const char *argv0)
 	fprintf(fp, "  -m, --musicpath=MUSIC         Specify root path to the music files\n");
 	fprintf(fp, "  -p, --playlistpath=PLAYLIST   Specify path to the playlist files\n");
 	fprintf(fp, "\n");
-#ifndef		_WIN32
 	fprintf(fp, "Language-specific options:\n");
+#ifndef		_WIN32
 	fprintf(fp, "  -e, --encoding=SE     Specify character encoding used by the operating system\n");
 	fprintf(fp, "  -w, --tagencoding=TE  Specify character encoding for parsing non-Unicode tags\n");
 #endif/*_WIN32*/
-#ifdef	SUPPORT_RTL
-	fprintf(fp, "  -l, --rtl             Store Arabic/Hebrew characters in right-to-left order\n");
-#endif/*SUPPORT_RTL*/
+	fprintf(fp, "  -b, --bidi            Arrange BiDi characters to be displayed correctly (exp)\n");
 	fprintf(fp, "\n");
 	fprintf(fp, "Miscellaneous options:\n");
 	fprintf(fp, "  -D, --dump            Output a database dump in plain text format\n");
@@ -196,6 +204,24 @@ static void easyh10_error(void *instance, int code, void *param)
 		ucs2tombs(mbs, MAX_PATH, (const ucs2_char_t*)param, ucs2len((const ucs2_char_t*)param)+1);
 		fprintf(fp, "WARNING: Failed to write playlist: %s\n", mbs);
 		break;
+
+	case EASYH10_WARNING_TUNER_TOOMANYENTRIES:
+		fprintf(fp, "WARNING: Too many entries for a preset in the player\n");
+		break;
+	case EASYH10_WARNING_TUNER_INVALIDLINE:
+		{
+			char *mbs = ucs2dupmbs((ucs2_char_t*)param);
+			fprintf(fp, "WARNING: Invalid line:\n  %s\n", mbs);
+			ucs2free(mbs);
+		}
+		break;
+	case EASYH10_WARNING_TUNER_INVALIDFREQUENCY:
+		{
+			char *mbs = ucs2dupmbs((ucs2_char_t*)param);
+			fprintf(fp, "WARNING: Invalid frequency specified:\n  %s\n", mbs);
+			ucs2free(mbs);
+		}
+		break;
 	}
 }
 
@@ -394,6 +420,51 @@ static int playlist_progress(void *instance, int phase, int param_int, void *par
 	return 0;
 }
 
+static int tuner_progress(void *instance, int phase, int param_int, void *param_ptr)
+{
+	progress_data_t* pd = (progress_data_t*)instance;
+	FILE *fp = pd->fp;
+
+	switch (phase) {
+	case EASYH10P_TUNER_IMPORT_START:
+		fprintf(fp, "Importing a tuner preset:\n");
+		break;
+	case EASYH10P_TUNER_IMPORT:
+		{
+			char *mbs = ucs2dupmbs(filepathutil_skippath((ucs2_char_t*)param_ptr));
+			fprintf(fp, "  %d presets in %s\n", param_int, mbs);
+			ucs2free(mbs);
+		}
+		break;
+	case EASYH10P_TUNER_IMPORT_MERGE:
+		fprintf(fp, "  %d presets were generated after merging.\n", param_int);
+		break;
+	case EASYH10P_TUNER_IMPORT_END:
+		fprintf(fp, "\n");
+		break;
+	case EASYH10P_TUNER_EXPORT_START:
+		fprintf(fp, "Exporting a tuner preset:\n");
+		break;
+	case EASYH10P_TUNER_EXPORT:
+		{
+			char *mbs = ucs2dupmbs((ucs2_char_t*)param_ptr);
+			fprintf(fp, "  %d presets in %s\n", param_int, mbs);
+			ucs2free(mbs);
+		}
+		break;
+	case EASYH10P_TUNER_EXPORT_MERGE:
+		fprintf(fp, "  %d presets were generated after merging.\n", param_int);
+		break;
+	case EASYH10P_TUNER_EXPORT_END:
+		fprintf(fp, "\n");
+		break;
+	}
+
+	pd->prev_phase = phase;
+	fflush(fp);
+	return 0;
+}
+
 int locate_template(ucs2_char_t* pathname, const char *_filename, const ucs2_char_t* path_to_root)
 {
 	if (_filename) {
@@ -570,6 +641,7 @@ int main(int argc, char *argv[])
 	ucs2_char_t path_to_music[MAX_PATH+1], path_to_playlist[MAX_PATH+1];
 	ucs2_char_t template_filename[MAX_PATH+1];
 	ucs2_char_t playlist_extension[MAX_PATH+1];
+	ucs2_char_t tuner_preset_import[MAX_PATH+1], tuner_preset_export[MAX_PATH];
 	const char *media_info_source = NULL;
 	FILE *fpi = stdin, *fpo = stdout, *fpe = stderr;
 	progress_data_t pd;
@@ -594,11 +666,11 @@ int main(int argc, char *argv[])
 
 	/* Set character encoding. */
 #ifndef		_WIN32
-	ucs2init(MBS_CHARSET);
-	if (set_encoding(MBS_CHARSET) != 0) {
+	ucs2init(SYSTEM_ENCODING);
+	if (set_encoding(SYSTEM_ENCODING) != 0) {
 		return 1;
 	}
-	if (set_encoding_music(MBS_CHARSET) != 0) {
+	if (set_encoding_music(MUSICTAG_ENCODING) != 0) {
 		return 1;
 	}
 #endif/*_WIN32*/
@@ -621,6 +693,11 @@ int main(int argc, char *argv[])
 			{"guess-missing",	no_argument,		0, 'g'},
 			{"find-missing",	no_argument,		0, 'f'},
 			{"skip-missing",	no_argument,		0, 'k'},
+			{"regex-playlist",	no_argument,		0, 'x'},
+			{"auto-shuffle",	no_argument,		0, 'l'},
+			{"import-tuner",	required_argument,	0, 'I'},
+			{"export-tuner",	required_argument,	0, 'E'},
+			{"merge-tuner",		no_argument,		0, 'a'},
 			{"model",			required_argument,	0, 't'},
 			{"gen-model",		required_argument,	0, 'T'},
 			{"install-model",	no_argument,		0, 'X'},
@@ -632,9 +709,7 @@ int main(int argc, char *argv[])
 			{"encoding",		required_argument,	0, 'e'},
 			{"tagencoding",		required_argument,	0, 'w'},
 		#endif/*_WIN32*/
-		#ifdef	SUPPORT_RTL
-			{"rtl",				no_argument,		0, 'l'},
-		#endif
+			{"bidi",			no_argument,		0, 'b'},
 			{"dump",			no_argument,		0, 'D'},
 			{"version",			no_argument,		0, 'V'},
 			{"help",			no_argument,		0, 'h'},
@@ -642,9 +717,9 @@ int main(int argc, char *argv[])
 			{NULL, 0, 0, 0}
 		};
 #ifndef	_WIN32
-		int c = getopt_long(argc, argv, "CnicUo:z:PMrgfkt:T:Xs:d:m:p:e:w:lDVhv", long_options, &option_index);
+		int c = getopt_long(argc, argv, "CnicUo:z:PMrgfkxlI:E:at:T:Xs:d:m:p:e:w:bDVhv", long_options, &option_index);
 #else
-		int c = getopt_long(argc, argv, "CnicUo:z:PMrgfkt:T:Xs:d:m:p:lDVhv", long_options, &option_index);
+		int c = getopt_long(argc, argv, "CnicUo:z:PMrgfkxlI:E:at:T:Xs:d:m:p:bDVhv", long_options, &option_index);
 #endif/*_WIN32*/
 		if (c == -1) {
 			break;
@@ -731,6 +806,26 @@ int main(int argc, char *argv[])
 		case 'k':
 			easyh10_flag |= EASYH10_PLAYLIST_SKIP_MISSING;
 			break;
+		case 'x':
+			easyh10_flag |= EASYH10_PLAYLIST_REGEX;
+			break;
+		case 'l':
+			easyh10_flag |= EASYH10_PLAYLIST_AUTO_SHUFFLE;
+			break;
+
+		case 'I':
+			mode |= MODE_IMPORT;
+			easyh10_flag |= EASYH10_TUNER_IMPORT;
+			mbstoucs2(tuner_preset_import, MAX_PATH, optarg, strlen(optarg));
+			break;
+		case 'E':
+			mode |= MODE_EXPORT;
+			easyh10_flag |= EASYH10_TUNER_EXPORT;
+			mbstoucs2(tuner_preset_export, MAX_PATH, optarg, strlen(optarg));
+			break;
+		case 'a':
+			easyh10_flag |= EASYH10_TUNER_MERGE;
+			break;
 
 		case 't':
 			ucs2free(templ);
@@ -763,10 +858,6 @@ int main(int argc, char *argv[])
 				ret = 1;
 				goto exit_main;
 			}
-			if (set_encoding_music(optarg) != 0) {
-				ret = 1;
-				goto exit_main;
-			}
 			break;
 		case 'w':
 			if (set_encoding_music(optarg) != 0) {
@@ -775,10 +866,8 @@ int main(int argc, char *argv[])
 			}
 			break;
 #endif/*_WIN32*/
-		case 'l':
-#ifdef	SUPPORT_RTL
+		case 'b':
 			easyh10_flag |= EASYH10_DATABASE_RIGHTTOLEFT;
-#endif
 			break;
 
 		case 'D':
@@ -873,14 +962,14 @@ int main(int argc, char *argv[])
 		/* Set path to the music. */
 		if (!*path_to_music) {
 			ucs2cpy(path_to_music, path_to_root);
-			filepathutil_addslash(path_to_db);
+			filepathutil_addslash(path_to_music);
 			ucs2cat(path_to_music, env.music_location);
 		}
 
 		/* Set path to the playlist. */
 		if (!*path_to_playlist) {
 			ucs2cpy(path_to_playlist, path_to_root);
-			filepathutil_addslash(path_to_db);
+			filepathutil_addslash(path_to_playlist);
 			ucs2cat(path_to_playlist, env.playlist_location);
 		}
 
@@ -906,7 +995,7 @@ int main(int argc, char *argv[])
 	}
 
 	/* make sure path_to_music. */
-	if ((mode & MODE_DATABASE) || (easyh10_flag & EASYH10_PLAYLIST_MUSIC) || (easyh10_flag & EASYH10_PLAYLIST_SEARCH_MISSING)) {
+	if ((mode & MODE_DATABASE) || (easyh10_flag & (EASYH10_PLAYLIST_MUSIC | EASYH10_PLAYLIST_SEARCH_MISSING | EASYH10_PLAYLIST_REGEX))) {
 		if (*path_to_music) {
 			char *mbs = NULL;
 			filepathutil_addslash(path_to_music);
@@ -1053,6 +1142,27 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	if ((mode & MODE_IMPORT) && (easyh10_flag & EASYH10_TUNER_IMPORT)) {
+		ret = easyh10_tuner_import(
+			path_to_system,
+			tuner_preset_import,
+			easyh10_flag,
+			&pd,
+			tuner_progress,
+			easyh10_error
+			);
+	}
+	if ((mode & MODE_EXPORT) && (easyh10_flag & EASYH10_TUNER_EXPORT)) {
+		ret = easyh10_tuner_export(
+			path_to_system,
+			tuner_preset_export,
+			easyh10_flag,
+			&pd,
+			tuner_progress,
+			easyh10_error
+			);
+	}
+
 	/* Dump the database. */
 	if (mode & MODE_DUMP) {
 		ret = easyh10_dump(
